/*
 * Host Monitor Utility
 *
 * Copyright (C) 2003, Jonathan Sevy <jsevy@mcs.drexel.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */



import java.util.*;
import java.net.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.io.*;
import java.math.*;
import snmp.*;




public class HostMonitor extends JFrame
							implements ActionListener, Runnable
{
	
	JButton newHostButton, newCommunityButton, scanButton;
	JTextArea messagesArea;
	JScrollPane messagesScroll, hostTableScroll;
	JTextField hostIDField, communityField;
	JLabel authorLabel, hostIDLabel, communityLabel;
	JTable hostTable;
	
	MenuBar theMenubar;
	Menu fileMenu;
	MenuItem quitItem, aboutItem;
	
	SNMPv1CommunicationInterface comInterface;
	String community;
	InetAddress hostAddress;
	int version;
	int wirelessHostNum = 1;
	
	Thread statusThread;
	
	Vector hostInfoVector, columnNamesVector;	
	
	// WindowCloseAdapter to catch window close-box closings
	private class WindowCloseAdapter extends WindowAdapter
	{ 
		public void windowClosing(WindowEvent evt)
		{
			// stop current statusThread, if alive, and wait for it to die:
			stopMonitoring();
			
			// then git!
			System.exit(0);
		}
	};
	
			
	
	public HostMonitor() 
	{
		hostInfoVector = new Vector();
		columnNamesVector = new Vector();
		columnNamesVector.add("MAC Address");
		columnNamesVector.add("Time Associated");
		
		setUpDisplay();
		
		try
		{
			version = 0;	// SNMPv1
			hostAddress = InetAddress.getByName(hostIDField.getText());
			community = communityField.getText();
			
			comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
			// set socket timeout to 5 seconds
			comInterface.setSocketTimeout(5000);
			
			statusThread = new Thread(this);
			
			
		}
		catch(Exception e)
		{
			messagesArea.setText("Exception during startup:  " + e + "\n");
		}
		
		
		
	}
	
	
	
	private void setUpDisplay()
	{
		
		
		try
		{
			// set look-and-feel to native platform l&f, if possible
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e)
		{
			// or not...
		}

		
		this.getRootPane().setBorder(new BevelBorder(BevelBorder.RAISED));
		
		// set fonts to smaller-than-normal size, for compaction!
		UIManager manager = new UIManager();
		FontUIResource appFont = new FontUIResource("SansSerif", Font.PLAIN, 10);
		UIDefaults defaults = manager.getLookAndFeelDefaults();
		Enumeration keys = defaults.keys();
		
		while (keys.hasMoreElements())
		{
			String nextKey = (String)(keys.nextElement());
			if ((nextKey.indexOf("font") > -1) || (nextKey.indexOf("Font") > -1))
			{
				manager.put(nextKey, appFont);
			}
		}
		
		
		// add WindowCloseAdapter to catch window close-box closings
		addWindowListener(new WindowCloseAdapter());

		
		theMenubar = new MenuBar();
		this.setMenuBar(theMenubar);
		fileMenu = new Menu("File");
		
		aboutItem = new MenuItem("About...");
		aboutItem.setActionCommand("about");
		aboutItem.addActionListener(this);
		fileMenu.add(aboutItem);
		
		fileMenu.addSeparator();
		
		quitItem = new MenuItem("Quit");
		quitItem.setActionCommand("quit");
		quitItem.addActionListener(this);
		fileMenu.add(quitItem);
		
		theMenubar.add(fileMenu);
		
		
		hostIDLabel = new JLabel("Base station address:");
		hostIDField = new JTextField(10);
		hostIDField.setText("10.0.1.1");
		hostIDField.setEditable(false);
		
		communityLabel = new JLabel("Community (password):");
		communityField = new JTextField(10);
		communityField.setText("");
		communityField.setEditable(false);
		
		authorLabel = new JLabel(" Version 2.0        J. Sevy, September 2003 ");
		authorLabel.setFont(new Font("SansSerif", Font.ITALIC, 8));
			
		
		newHostButton = new JButton("  Set address  ");
		newHostButton.setActionCommand("new host");
		newHostButton.addActionListener(this);
		
		newCommunityButton = new JButton(" Set community ");
		newCommunityButton.setActionCommand("new community");
		newCommunityButton.addActionListener(this);
		
		scanButton = new JButton("Start monitoring");
		scanButton.setActionCommand("start monitoring");
		scanButton.addActionListener(this);
		
		hostTable = new JTable(hostInfoVector, columnNamesVector);
		hostTable.setPreferredScrollableViewportSize(new Dimension(450,150));
		hostTableScroll = new JScrollPane(hostTable);
		
		messagesArea = new JTextArea(3,50);
		messagesScroll = new JScrollPane(messagesArea);
		 
		/*
        URL url = AirportBaseStationHangup.class.getResource("iconImage.gif");
		this.setIconImage(Toolkit.getDefaultToolkit().getImage(url));
		*/
		
		
		// now set up display
		
		// set params for layout manager
		GridBagLayout  theLayout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.NONE;
		c.ipadx = 0;
		c.ipady = 0;
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.CENTER;
		c.weightx = 0;
		c.weighty = 0;
		
		
		this.setTitle("Access Point Host Monitoring Utility");
		
		
		Panel hostPanel = new Panel();
		hostPanel.setLayout(theLayout);
		
		c.gridx = 1;
		c.gridy = 1;
		theLayout.setConstraints(hostIDLabel, c);
		hostPanel.add(hostIDLabel);
		
		c.gridx = 2;
		c.gridy = 1;
		theLayout.setConstraints(hostIDField, c);
		hostPanel.add(hostIDField);
		
		c.gridx = 3;
		c.gridy = 1;
		theLayout.setConstraints(newHostButton, c);
		hostPanel.add(newHostButton);
		
		c.gridx = 1;
		c.gridy = 2;
		theLayout.setConstraints(communityLabel, c);
		hostPanel.add(communityLabel);
		
		c.gridx = 2;
		c.gridy = 2;
		theLayout.setConstraints(communityField, c);
		hostPanel.add(communityField);
		
		c.gridx = 3;
		c.gridy = 2;
		theLayout.setConstraints(newCommunityButton, c);
		hostPanel.add(newCommunityButton);
		
		c.gridwidth = 3;
		c.gridx = 1;
		c.gridy = 3;
		theLayout.setConstraints(scanButton, c);
		hostPanel.add(scanButton);
		
		c.gridwidth = 1;
		
		
		this.getContentPane().setLayout(theLayout);
		
		
		c.gridx = 1;
		c.gridy = 1;
		theLayout.setConstraints(hostPanel, c);
		this.getContentPane().add(hostPanel);
		
		c.gridx = 1;
		c.gridy = 2;
		JLabel messagesLabel = new JLabel("Attached wireless hosts");
		theLayout.setConstraints(messagesLabel, c);
		this.getContentPane().add(messagesLabel);
		
		c.fill = GridBagConstraints.BOTH;
		c.weightx = .5;
		c.weighty = .5;
		c.gridx = 1;
		c.gridy = 3;
		theLayout.setConstraints(hostTableScroll, c);
		this.getContentPane().add(hostTableScroll);				
		
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.weighty = 0;
		c.gridx = 1;
		c.gridy = 4;
		messagesLabel = new JLabel("Messages");
		theLayout.setConstraints(messagesLabel, c);
		this.getContentPane().add(messagesLabel);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = .5;
		c.weighty = 0;
		c.gridx = 1;
		c.gridy = 5;
		theLayout.setConstraints(messagesScroll, c);
		this.getContentPane().add(messagesScroll);				
		
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.weighty = 0;
		c.gridx = 1;
		c.gridy = 6;
		theLayout.setConstraints(authorLabel, c);
		this.getContentPane().add(authorLabel);
		
		
	}
	
	
	
	
	
	public void actionPerformed(ActionEvent theEvent)
	// respond to button pushes, menu selections
	{
		String command = theEvent.getActionCommand();
		
	
		if (command == "quit")
		{
			// stop current link test thread, and wait for it to die:
			stopMonitoring();
			
			System.exit(0);
		}
		
		
		
		if (command == "about")
		{
			AboutDialog aboutDialog = new AboutDialog(this);
		}
		
		
		
		if (command == "new host")
		{
			
			String newHost = JOptionPane.showInputDialog("Input access point address:");
			if (newHost != null)
			{
						
				try
				{
					version = 0;	// SNMPv1
					hostAddress = InetAddress.getByName(newHost);
			
					// stop current link test thread, and wait for it to die:
					stopMonitoring();
					
					// set up polling parameters
					comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
					// set socket timeout to 5 seconds
					comInterface.setSocketTimeout(5000);
					
					hostIDField.setText(newHost);					
				
				}
				catch(UnknownHostException e)
				{
					JOptionPane.showMessageDialog(this, "Unknown host supplied.");
				}
				catch(Exception e)
				{
					JOptionPane.showMessageDialog(this, "Error setting new access point address: " + e);
				}
				
			}
			
		}
		
		
		
		if (command == "new community")
		{
			
			String newCommunity = JOptionPane.showInputDialog("Input new community:");
			
			if (newCommunity != null)
			{
				try
				{
					version = 0;	// SNMPv1
					community = newCommunity;
					
					// stop current link test thread, and wait for it to die:
					stopMonitoring();
					
					// set up polling parameters
					comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
					// set socket timeout to 5 seconds
					comInterface.setSocketTimeout(5000);
					
					communityField.setText(newCommunity);					
				
				}
				catch(Exception e)
				{
					JOptionPane.showMessageDialog(this, "Error setting new community: " + e);
				}
				
			}
			
		}
		
		
		
		if (command == "start monitoring")
		{
			startMonitoring();
		}
		
		
		
		if (command == "stop monitoring")
		{
			stopMonitoring();
		}
		
		
	}
	
	
	
	
	private void stopMonitoring()
	{
		if (statusThread.isAlive())
		{
			try
			{
				// stop current statusThread, and wait for it to die:
				statusThread.interrupt();
				statusThread.join();
			}
			catch(InterruptedException ex)
			{
				messagesArea.setText("Exception during monitoring thread exit:  " + ex + "\n");
			}
			
			scanButton.setText("Start monitoring");
			scanButton.setActionCommand("start monitoring");
		
		}			
	}
	
	
	
	private void startMonitoring()
	{
		if (!statusThread.isAlive())
		{
			scanButton.setText("Stop monitoring");
			scanButton.setActionCommand("stop monitoring");
		
			// recreate and start status polling thread
			statusThread = new Thread(this);
			statusThread.start();
		}			
	}
	
	
	
	
	private String hexByte(byte b)
	{
		int pos = b;
		if (pos < 0)
			pos += 256;
		String returnString = new String();
		returnString += Integer.toHexString(pos/16);
		returnString += Integer.toHexString(pos%16);
		return returnString;
	}
	
	
	
	
	
	public void run()
	{
		
		String[] baseIDs = {"1.3.6.1.4.1.63.501.3.2.2.1.1", "1.3.6.1.4.1.63.501.3.2.2.1.4"};
		
		
		while(!Thread.interrupted())
		{
		
			try
			{
				// clear messages area
				messagesArea.setText("");
				
				// clear host info vector
				hostInfoVector.removeAllElements();
				
				// do wireless host discovery process
				String itemID;
				SNMPVarBindList newVars;
				SNMPSequence pair;
				SNMPInteger snmpIntegerValue;
				SNMPObject snmpValue;

                // retrieve all the SNMP table entries
                newVars = comInterface.retrieveMIBTable(baseIDs);
                
				
				for (int i = 0; i < newVars.size(); i += 2)
				{
					// use TreeSet so elements will be sorted
					TreeSet treeSet = new TreeSet();
					
					// get MAC address, time associated
					
					HostInfo hostInfo = new HostInfo();
					
					pair = (SNMPSequence)(newVars.getSNMPObjectAt(i));
					snmpValue = pair.getSNMPObjectAt(1);
					SNMPOctetString snmpNSAPValue = (SNMPOctetString)pair.getSNMPObjectAt(1);
					String hostMacAddress = snmpNSAPValue.toHexString();
					hostInfo.add(hostMacAddress);
					
					pair = (SNMPSequence)(newVars.getSNMPObjectAt(i+1));
					snmpValue = pair.getSNMPObjectAt(1);
					String hostTimeAssociated = snmpValue.toString();
					hostInfo.add(hostTimeAssociated);
					
					treeSet.add(hostInfo);
					
					// transfer elements to host info vector associated with table
					hostInfoVector.addAll(treeSet);
				}
				
				hostTable.tableChanged(new TableModelEvent(hostTable.getModel()));	
			
			}
			catch(Exception e)
			{
				messagesArea.setText("Exception during base station monitoring:  " + e + "\n" + "Check community name.");
			}
			
			
			try
			{
				// sleep for 5 seconds
				Thread.currentThread().sleep(5000);
			}
			catch(InterruptedException e)
			{
				// don't bother informing of interruption; will just exit...
				// but must reset this thread's interrupted status
				statusThread.interrupt();
				//messagesArea.setText("Exception during modem status polling:  " + e + "\n");
			}
		
		}
		
			
		//messagesArea.setText("Status thread exiting!\n");
		
	}
	

	
	
	
	public static void main(String args[]) 
	{
		try
		{
			HostMonitor theApp = new HostMonitor();
			theApp.pack();
			
			// tweak app size to make it a little larger than necessary, to address the
			// "shrunken textfields" problem arising from the layout manager packing stuff
			// a little too tightly.
			Dimension dim = theApp.getSize();
			dim.height += 20;
			dim.width += 20;
			theApp.setSize(dim);
			
			theApp.show();
		}
		catch (Exception e)
		{}
	}
	

}